package javax.management.remote.rmi;

import java.util.HashSet;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.RMIClientSocketFactory;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import com.sun.jmx.remote.internal.IIOPHelper;
import java.util.Hashtable;
import javax.management.remote.MBeanServerForwarder;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.management.InstanceNotFoundException;
import com.sun.jmx.remote.security.MBeanServerFileAccessController;
import java.util.HashMap;
import javax.management.remote.JMXConnector;
import java.util.Collections;
import com.sun.jmx.remote.util.EnvHelp;
import java.net.MalformedURLException;
import java.io.IOException;
import javax.management.MBeanServer;
import java.util.Set;
import java.util.Map;
import javax.management.remote.JMXServiceURL;
import com.sun.jmx.remote.util.ClassLogger;
import javax.management.remote.JMXConnectorServer;

public class RMIConnectorServer extends JMXConnectorServer
{
    public static final String JNDI_REBIND_ATTRIBUTE = "jmx.remote.jndi.rebind";
    public static final String RMI_CLIENT_SOCKET_FACTORY_ATTRIBUTE = "jmx.remote.rmi.client.socket.factory";
    public static final String RMI_SERVER_SOCKET_FACTORY_ATTRIBUTE = "jmx.remote.rmi.server.socket.factory";
    private static final char[] intToAlpha;
    private static ClassLogger logger;
    private JMXServiceURL address;
    private RMIServerImpl rmiServerImpl;
    private final Map<String, ?> attributes;
    private ClassLoader defaultClassLoader;
    private String boundJndiUrl;
    private static final int CREATED = 0;
    private static final int STARTED = 1;
    private static final int STOPPED = 2;
    private int state;
    private static final Set<RMIConnectorServer> openedServers;
    
    public RMIConnectorServer(JMXServiceURL jmxServiceURL, Map<String, ?> map) throws IOException {
        this(jmxServiceURL, map, (MBeanServer)null);
    }
    
    public RMIConnectorServer(JMXServiceURL jmxServiceURL, Map<String, ?> map, MBeanServer mBeanServer) throws IOException {
        this(jmxServiceURL, map, (RMIServerImpl)null, mBeanServer);
    }
    
    public RMIConnectorServer(JMXServiceURL address, Map<String, ?> map, RMIServerImpl rmiServerImpl, MBeanServer mbeanServer) throws IOException {
        super(mbeanServer);
        this.defaultClassLoader = null;
        this.state = 0;
        if (address == null) {
            throw new IllegalArgumentException("Null JMXServiceURL");
        }
        if (rmiServerImpl == null) {
            String protocol = address.getProtocol();
            if (protocol == null || (!protocol.equals("rmi") && !protocol.equals("iiop"))) {
                throw new MalformedURLException("Invalid protocol type: " + protocol);
            }
            String urlPath = address.getURLPath();
            if (!urlPath.equals("") && !urlPath.equals("/") && !urlPath.startsWith("/jndi/")) {
                throw new MalformedURLException("URL path must be empty or start with /jndi/");
            }
        }
        if (map == null) {
            this.attributes = Collections.emptyMap();
        }
        else {
            EnvHelp.checkAttributes(map);
            this.attributes = Collections.unmodifiableMap(map);
        }
        this.address = address;
        this.rmiServerImpl = rmiServerImpl;
        return;
    }
    
    @Override
    public JMXConnector toJMXConnector(Map<String, ?> attributes) throws IOException {
        if (!this.isActive()) {
            throw new IllegalStateException("Connector is not active");
        }
        HashMap attributes2 = new HashMap((this.attributes == null) ? Collections.emptyMap() : this.attributes);
        if (attributes != null) {
            EnvHelp.checkAttributes(attributes);
            attributes2.putAll(attributes);
        }
        return new RMIConnector((RMIServer)this.rmiServerImpl.toStub(), EnvHelp.filterAttributes(attributes2));
    }
    
    @Override
    public synchronized void start() throws IOException {
        int traceOn = RMIConnectorServer.logger.traceOn();
        if (this.state == 1) {
            if (!(traceOn == 0)) {
                RMIConnectorServer.logger.trace("start", "already started");
            }
            return;
        }
        else if (this.state == 2) {
            if (!(traceOn == 0)) {
                RMIConnectorServer.logger.trace("start", "already stopped");
            }
            throw new IOException("The server has been stopped.");
        }
        else {
            if (this.getMBeanServer() == null) {
                throw new IllegalStateException("This connector server is not attached to an MBean server");
            }
            if (this.attributes != null) {
                String accessFileName = (String)this.attributes.get("jmx.remote.x.access.file");
                if (accessFileName != null) {
                    MBeanServerFileAccessController mBeanServerForwarder;
                    try {
                        mBeanServerForwarder = new MBeanServerFileAccessController(accessFileName);
                    }
                    catch (IOException cause) {
                        throw (IllegalArgumentException)EnvHelp.initCause(new IllegalArgumentException(cause.getMessage()), cause);
                    }
                    this.setMBeanServerForwarder(mBeanServerForwarder);
                }
            }
            try {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("start", "setting default class loader");
                }
                this.defaultClassLoader = EnvHelp.resolveServerClassLoader(this.attributes, this.getMBeanServer());
            }
            catch (InstanceNotFoundException ex) {
                throw (IllegalArgumentException)EnvHelp.initCause(new IllegalArgumentException("ClassLoader not found: " + ex), ex);
            }
            if (!(traceOn == 0)) {
                RMIConnectorServer.logger.trace("start", "setting RMIServer object");
            }
            RMIServerImpl rmiServerImpl;
            if (this.rmiServerImpl != null) {
                rmiServerImpl = this.rmiServerImpl;
            }
            else {
                rmiServerImpl = this.newServer();
            }
            rmiServerImpl.setMBeanServer(this.getMBeanServer());
            rmiServerImpl.setDefaultClassLoader(this.defaultClassLoader);
            rmiServerImpl.setRMIConnectorServer(this);
            rmiServerImpl.export();
            try {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("start", "getting RMIServer object to export");
                }
                RMIServer objectToBind = objectToBind(rmiServerImpl, this.attributes);
                if (this.address != null && this.address.getURLPath().startsWith("/jndi/")) {
                    String substring = this.address.getURLPath().substring(6);
                    if (!(traceOn == 0)) {
                        RMIConnectorServer.logger.trace("start", "Using external directory: " + substring);
                    }
                    int computeBooleanFromString = EnvHelp.computeBooleanFromString((String)this.attributes.get("jmx.remote.jndi.rebind"));
                    if (!(traceOn == 0)) {
                        RMIConnectorServer.logger.trace("start", "jmx.remote.jndi.rebind=" + (boolean)computeBooleanFromString);
                    }
                    try {
                        if (!(traceOn == 0)) {
                            RMIConnectorServer.logger.trace("start", "binding to " + substring);
                        }
                        this.bind(substring, EnvHelp.mapToHashtable(this.attributes), objectToBind, (boolean)computeBooleanFromString);
                        this.boundJndiUrl = substring;
                    }
                    catch (NamingException obj) {
                        throw newIOException("Cannot bind to URL [" + substring + "]: " + obj, obj);
                    }
                }
                else {
                    if (!(traceOn == 0)) {
                        RMIConnectorServer.logger.trace("start", "Encoding URL");
                    }
                    this.encodeStubInAddress(objectToBind, this.attributes);
                    if (!(traceOn == 0)) {
                        RMIConnectorServer.logger.trace("start", "Encoded URL: " + this.address);
                    }
                }
            }
            catch (Exception obj2) {
                try {
                    rmiServerImpl.close();
                }
                catch (Exception ex2) {}
                if ((obj2 instanceof RuntimeException)) {
                    throw (RuntimeException)obj2;
                }
                if ((obj2 instanceof IOException)) {
                    throw (IOException)obj2;
                }
                throw newIOException("Got unexpected exception while starting the connector server: " + obj2, obj2);
            }
            this.rmiServerImpl = rmiServerImpl;
            synchronized (RMIConnectorServer.openedServers) {
                RMIConnectorServer.openedServers.add(this);
            }
            this.state = 1;
            if (!(traceOn == 0)) {
                RMIConnectorServer.logger.trace("start", "Connector Server Address = " + this.address);
                RMIConnectorServer.logger.trace("start", "started.");
            }
            return;
        }
    }
    
    @Override
    public void stop() throws IOException {
        int traceOn = RMIConnectorServer.logger.traceOn();
        synchronized (this) {
            if (this.state == 2) {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "already stopped.");
                }
                return;
            }
            else {
                if (this.state == 0 && traceOn) {
                    RMIConnectorServer.logger.trace("stop", "not started yet.");
                }
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "stopping.");
                }
                this.state = 2;
            }
        }
        synchronized (RMIConnectorServer.openedServers) {
            RMIConnectorServer.openedServers.remove(this);
        }
        IOException ioException = null;
        if (this.rmiServerImpl != null) {
            try {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "closing RMI server.");
                }
                this.rmiServerImpl.close();
            }
            catch (IOException ex) {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "failed to close RMI server: " + ex);
                }
                if (RMIConnectorServer.logger.debugOn()) {
                    RMIConnectorServer.logger.debug("stop", ex);
                }
                ioException = ex;
            }
        }
        if (this.boundJndiUrl != null) {
            try {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "unbind from external directory: " + this.boundJndiUrl);
                }
                InitialContext o = new InitialContext(EnvHelp.mapToHashtable(this.attributes));
                o.unbind(this.boundJndiUrl);
                o.close();
            }
            catch (NamingException obj) {
                if (!(traceOn == 0)) {
                    RMIConnectorServer.logger.trace("stop", "failed to unbind RMI server: " + obj);
                }
                if (RMIConnectorServer.logger.debugOn()) {
                    RMIConnectorServer.logger.debug("stop", obj);
                }
                if (ioException == null) {
                    ioException = newIOException("Cannot bind to URL: " + obj, obj);
                }
            }
        }
        if (ioException != null) {
            throw ioException;
        }
        if (!(traceOn == 0)) {
            RMIConnectorServer.logger.trace("stop", "stopped");
        }
        return;
    }
    
    @Override
    public synchronized boolean isActive() {
        return this.state == 1;
    }
    
    @Override
    public JMXServiceURL getAddress() {
        if (!this.isActive()) {
            return null;
        }
        return this.address;
    }
    
    @Override
    public Map<String, ?> getAttributes() {
        return Collections.unmodifiableMap(EnvHelp.filterAttributes(this.attributes));
    }
    
    @Override
    public synchronized void setMBeanServerForwarder(MBeanServerForwarder mBeanServerForwarder) {
        super.setMBeanServerForwarder(mBeanServerForwarder);
        if (this.rmiServerImpl != null) {
            this.rmiServerImpl.setMBeanServer(this.getMBeanServer());
        }
    }
    
    @Override
    protected void connectionOpened(String connectionId, String message, Object userData) {
        super.connectionOpened(connectionId, message, userData);
    }
    
    @Override
    protected void connectionClosed(String connectionId, String message, Object userData) {
        super.connectionClosed(connectionId, message, userData);
    }
    
    @Override
    protected void connectionFailed(String connectionId, String message, Object userData) {
        super.connectionFailed(connectionId, message, userData);
    }
    
    void bind(String s, Hashtable<?, ?> environment, RMIServer rmiServer, boolean b) throws NamingException, MalformedURLException {
        InitialContext initialContext = new InitialContext(environment);
        if (b) {
            initialContext.rebind(s, rmiServer);
        }
        else {
            initialContext.bind(s, rmiServer);
        }
        initialContext.close();
    }
    
    RMIServerImpl newServer() throws IOException {
        int iiopURL = isIiopURL(this.address, true);
        int port;
        if (this.address == null) {
            port = 0;
        }
        else {
            port = this.address.getPort();
        }
        if (!(iiopURL == 0)) {
            return newIIOPServer(this.attributes);
        }
        return newJRMPServer(this.attributes, port);
    }
    
    private void encodeStubInAddress(RMIServer rmiServer, Map<String, ?> map) throws IOException {
        String protocol;
        String host;
        int port;
        if (this.address == null) {
            if (IIOPHelper.isStub(rmiServer)) {
                protocol = "iiop";
            }
            else {
                protocol = "rmi";
            }
            host = null;
            port = 0;
        }
        else {
            protocol = this.address.getProtocol();
            host = (this.address.getHost().equals("") ? null : this.address.getHost());
            port = this.address.getPort();
        }
        this.address = new JMXServiceURL(protocol, host, port, encodeStub(rmiServer, map));
    }
    
    static boolean isIiopURL(JMXServiceURL jmxServiceURL, boolean b) throws MalformedURLException {
        String protocol = jmxServiceURL.getProtocol();
        if (protocol.equals("rmi")) {
            return false;
        }
        if (protocol.equals("iiop")) {
            return true;
        }
        if (b) {
            throw new MalformedURLException("URL must have protocol \"rmi\" or \"iiop\": \"" + protocol + "\"");
        }
        return false;
    }
    
    static String encodeStub(RMIServer rmiServer, Map<String, ?> map) throws IOException {
        if (IIOPHelper.isStub(rmiServer)) {
            return "/ior/" + encodeIIOPStub(rmiServer, map);
        }
        return "/stub/" + encodeJRMPStub(rmiServer, map);
    }
    
    static String encodeJRMPStub(RMIServer obj, Map<String, ?> map) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ObjectOutputStream objectOutputStream = new ObjectOutputStream(out);
        objectOutputStream.writeObject(obj);
        objectOutputStream.close();
        return byteArrayToBase64(out.toByteArray());
    }
    
    static String encodeIIOPStub(RMIServer rmiServer, Map<String, ?> map) throws IOException {
        try {
            return IIOPHelper.objectToString(IIOPHelper.getOrb(rmiServer), rmiServer);
        }
        catch (RuntimeException ex) {
            throw newIOException(ex.getMessage(), ex);
        }
    }
    
    private static RMIServer objectToBind(RMIServerImpl rmiServerImpl, Map<String, ?> map) throws IOException {
        return RMIConnector.connectStub((RMIServer)rmiServerImpl.toStub(), map);
    }
    
    private static RMIServerImpl newJRMPServer(Map<String, ?> map, int n) throws IOException {
        return new RMIJRMPServerImpl(n, (RMIClientSocketFactory)map.get("jmx.remote.rmi.client.socket.factory"), (RMIServerSocketFactory)map.get("jmx.remote.rmi.server.socket.factory"), map);
    }
    
    private static RMIServerImpl newIIOPServer(Map<String, ?> map) throws IOException {
        return new RMIIIOPServerImpl(map);
    }
    
    private static String byteArrayToBase64(byte[] array) {
        int length = array.length;
        int n = length / 3;
        int n2 = length - 3 * n;
        StringBuilder sb = new StringBuilder(4 * ((length + 2) / 3));
        int n3 = 0;
        for (int i = 0; i < n; ++i) {
            int n4 = array[n3++] & 0xFF;
            int n5 = array[n3++] & 0xFF;
            int n6 = array[n3++] & 0xFF;
            sb.append(RMIConnectorServer.intToAlpha[n4 >> 2]);
            sb.append(RMIConnectorServer.intToAlpha[(n4 << 4 & 0x3F) | n5 >> 4]);
            sb.append(RMIConnectorServer.intToAlpha[(n5 << 2 & 0x3F) | n6 >> 6]);
            sb.append(RMIConnectorServer.intToAlpha[n6 & 0x3F]);
        }
        if (n2 != 0) {
            int n7 = array[n3++] & 0xFF;
            sb.append(RMIConnectorServer.intToAlpha[n7 >> 2]);
            if (n2 == 1) {
                sb.append(RMIConnectorServer.intToAlpha[n7 << 4 & 0x3F]);
                sb.append("==");
            }
            else {
                int n8 = array[n3++] & 0xFF;
                sb.append(RMIConnectorServer.intToAlpha[(n7 << 4 & 0x3F) | n8 >> 4]);
                sb.append(RMIConnectorServer.intToAlpha[n8 << 2 & 0x3F]);
                sb.append('=');
            }
        }
        return sb.toString();
    }
    
    private static IOException newIOException(String message, Throwable cause) {
        return (IOException)EnvHelp.initCause(new IOException(message), cause);
    }
    
    static {
        intToAlpha = new char[] { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/' };
        RMIConnectorServer.logger = new ClassLogger("javax.management.remote.rmi", "RMIConnectorServer");
        openedServers = new HashSet();
    }
}
